from .base import AutoPackagerRunner
from .javascript import JavaScriptRunner
from .python_poetry import PythonPoetryRunner

__all__ = (
    "AutoPackagerRunner",
    "JavaScriptRunner",
    "PythonPoetryRunner",
)



from pathlib import Path
from typing import Protocol

from ...models import Noofile


class AutoPackagerRunner(Protocol):
    def __init__(self, location: Path) -> None:
        ...

    def package(self) -> Noofile:
        ...

    @staticmethod
    def detect(location: Path) -> bool:
        ...



from pathlib import Path

from tomli import loads

from ...models import Noofile, ReadVariable, ReplaceAction, Step
from ..reader import read


class PythonPoetryRunner:
    def __init__(self, location: Path) -> None:
        self.location = location

    def package(self) -> Noofile:
        pyproject = self.location / "pyproject.toml"
        pyproject_data = loads(pyproject.read_text())

        name = pyproject_data["tool"]["poetry"]["name"]
        author = pyproject_data["tool"]["poetry"]["authors"][0]
        desc = pyproject_data["tool"]["poetry"]["description"]
        repo = pyproject_data["tool"]["poetry"]["repository"]
        remote = read("Remote slug: ")

        noofile = Noofile(
            name=name.title(),
            remote=remote,
        )

        poetry_step = Step(name="Update Poetry")

        actions = [
            ReplaceAction(
                action="replace",
                files=["pyproject.toml"],
                src=f'"name" = "{name}"',
                dest=f'"name" = "$$noo:name"',
            ),
            ReplaceAction(
                action="replace",
                files=["pyproject.toml"],
                src=repo,
                dest=f"$$var:repository",
            ),
            ReplaceAction(
                action="replace",
                files=["pyproject.toml"],
                src=author,
                dest=f"$$var:author <$$var:email>",
            ),
            ReplaceAction(
                action="replace",
                files=["pyproject.toml"],
                src=f'"{desc}"',
                dest=f'"$$var:description"',
            ),
        ]

        poetry_step.actions.extend(actions)
        noofile.steps.append(poetry_step)

        noofile.read = [
            ReadVariable(
                name="repository",
                prompt="Enter repository URL",
            ),
            ReadVariable(
                name="author",
                prompt="Enter author name",
            ),
            ReadVariable(
                name="description",
                prompt="Enter description",
            ),
            ReadVariable(
                name="email",
                prompt="Enter email",
            ),
        ]

        return noofile

    @staticmethod
    def detect(location: Path) -> bool:
        path = location / "pyproject.toml"

        if not path.exists():
            return False

        content = path.read_text()

        return "[tool.poetry]" in content



from typing import Literal, Optional

from pydantic import BaseModel

from .actions import ActionType, List


class ReadVariable(BaseModel):
    name: str
    prompt: Optional[str] = None
    default: Optional[str] = None


class Condition(BaseModel):
    op: Literal["eq", "ne", "gt", "ge", "lt", "le"]
    var: str
    value: str


class Step(BaseModel):
    name: str
    actions: List[ActionType] = []
    conditions: Optional[List[Condition]] = None


class Noofile(BaseModel):
    name: Optional[str] = None
    remote: Optional[str] = None
    read: List[ReadVariable] = []
    steps: List[Step] = []



from pathlib import Path
from shutil import rmtree

from typer import Typer, echo

from ..impl.core import NooCore
from ..impl.packager import Packager
from .components import collection_app, registry_app
from .components.registry import reg

app = Typer()

app.add_typer(registry_app, name="reg")
app.add_typer(collection_app, name="collection")


@app.command("clone")
def clone(name: str, ref: str, dest: str = ".") -> None:
    path = Path(dest) / name

    if path.exists():
        echo(f"Path {path} already exists.")
        return

    try:
        abs_ref = reg.get_item(ref)
    except KeyError:
        abs_ref = ref

    echo(f"Cloning project {abs_ref} to {path}...")

    try:
        core = NooCore()
        core.clone(name, str(abs_ref), path)

        echo(f"Done!\n  cd {path}")
    except Exception as e:
        echo(f"An error occurred while cloning: {e}")
        rmtree(path)


@app.command("mod")
def mod(ref: str, dest: str = ".") -> None:
    path = Path(dest)

    if not path.exists():
        echo(f"Path {path} does not exist.")
        return

    try:
        abs_ref = reg.get_item(ref)
    except KeyError:
        abs_ref = ref

    echo(f"Modifying project with {abs_ref}...")

    core = NooCore()
    core.mod(str(abs_ref), path)

    echo(f"Done!\n  cd {path}")


@app.command("autopackage")
def autopackage(path: str = ".", dest: str = "./noofile.yml") -> None:
    _path = Path(path)
    _dest = Path(dest)

    if _dest.exists():
        echo(f"File {_dest} already exists.")
        return

    echo(f"Autopacking {_path} to {_dest}...")

    packager = Packager(_path)
    packager.package(_dest)

    echo(f"Done!")



from __future__ import annotations

from pathlib import Path

from typer import echo

from ..models import CreateAction, DeleteAction, RenameAction, ReplaceAction, Step
from .formatter import format_vars, replace

OPMAP = {
    "eq": lambda a, b: str(a) == str(b),
    "ne": lambda a, b: str(a) != str(b),
    "gt": lambda a, b: int(a) > int(b),
    "ge": lambda a, b: int(a) >= int(b),
    "lt": lambda a, b: int(a) < int(b),
    "le": lambda a, b: int(a) <= int(b),
}


class Runner:
    def __init__(self, base: Path, steps: list[Step], variables: dict[str, dict[str, str | int]]) -> None:
        self.base = base
        self.steps = steps
        self.vars = variables

    def _resolve_var(self, var: str) -> str | int:
        ns, name = var.removeprefix("$$").split(":", 1)

        if ns not in ("noo", "var"):
            raise ValueError(f"Unknown namespace: {ns}")

        if name not in self.vars[ns]:
            raise ValueError(f"Unknown variable: {ns}:{name}")

        return self.vars[ns][name]

    def _run_replace(self, files: list[str], src: str, dest: str) -> None:
        for file in files:
            path = self.base / file

            source = path.read_text()
            target = replace(source, src, dest, self.vars)
            path.write_text(target)

    def _run_delete(self, files: list[str]) -> None:
        for file in files:
            path = self.base / file

            if path.exists():
                path.unlink()

    def _run_create(self, file: str, content: str) -> None:
        path = self.base / file
        path.write_text(format_vars(content, self.vars))

    def _run_rename(self, file: str, dest: str) -> None:
        path = self.base / file

        if not path.exists():
            raise ValueError(f"No such file: {path}")

        path.rename(self.base / dest)

    def _verify_step_conditions(self, step: Step) -> bool:
        if step.conditions is None:
            return True

        for condition in step.conditions:
            var = self._resolve_var(condition.var)
            value = condition.value

            op = OPMAP[condition.op]

            if not op(var, value):
                return False

        return True

    def _run_step(self, step: Step) -> None:
        if not self._verify_step_conditions(step):
            echo(f"Skipping step {step.name}.")
            return

        for action in step.actions:
            if isinstance(action, ReplaceAction):
                self._run_replace(action.files, action.src, action.dest)
            elif isinstance(action, DeleteAction):
                self._run_delete(action.files)
            elif isinstance(action, CreateAction):
                self._run_create(action.file, action.content or "")
            elif isinstance(action, RenameAction):
                self._run_rename(action.file, action.dest)
            else:
                raise ValueError(f"Invalid action: {action}")

    def run(self) -> None:
        for step in self.steps:
            echo(f"Running step {step.name}.")

            self._run_step(step)



from pathlib import Path
from typing import List, Optional

from typer import Typer, echo

app = Typer()


@app.command("generate")
def generate(directory: str = ".", suffix: str = ".noofile.yml", remote: Optional[str] = None) -> None:
    path = Path(directory)

    if not path.exists():
        echo("The specified directory does not exist.")
        return

    if not path.is_dir():
        echo("The specified path is not a directory.")
        return

    files: List[Path] = []

    for file in path.iterdir():
        if not file.is_file():
            continue

        if not file.name.endswith(suffix):
            continue

        files.append(file)

    if not files:
        echo("No noofiles found.")
        return

    commands = []

    branch = "master"

    if remote and "@" in remote:
        remote, branch = remote.split("@")

    for file in files:
        name = file.name.split(".")[0]

        if remote:
            commands.append(f"noo reg add {name} https://raw.githubusercontent.com/{remote}/{branch}/{str(file)}")
        else:
            commands.append(f"noo reg add {name} {str(file)}")

    Path("register.sh").write_text("\n".join(commands))



from json import loads
from pathlib import Path

from ...models import Noofile, ReadVariable, ReplaceAction, Step
from ..reader import read


class JavaScriptRunner:
    def __init__(self, location: Path) -> None:
        self.location = location

    def package(self) -> Noofile:
        package = self.location / "package.json"
        package_data = loads(package.read_text())

        name = package_data["name"]
        author = package_data["author"]
        desc = package_data["description"]
        remote = read("Remote slug: ")

        noofile = Noofile(
            name=name.title(),
            remote=remote,
        )

        poetry_step = Step(name="Update package.json")

        actions = [
            ReplaceAction(
                action="replace",
                files=["package.json"],
                src=f'"name": "{name}"',
                dest=f'"name": "$$noo:name"',
            ),
            ReplaceAction(
                action="replace",
                files=["package.json"],
                src=author,
                dest=f"$$var:author <$$var:email>",
            ),
            ReplaceAction(
                action="replace",
                files=["package.json"],
                src=f'"{desc}"',
                dest=f'"$$var:description"',
            ),
        ]

        poetry_step.actions.extend(actions)
        noofile.steps.append(poetry_step)

        noofile.read = [
            ReadVariable(
                name="author",
                prompt="Enter author name",
            ),
            ReadVariable(
                name="description",
                prompt="Enter description",
            ),
            ReadVariable(
                name="email",
                prompt="Enter email",
            ),
        ]

        return noofile

    @staticmethod
    def detect(location: Path) -> bool:
        path = location / "package.json"

        return path.exists()



from json import dumps, loads
from os import getenv
from pathlib import Path

from typer import Typer, echo

from ...impl.utils import Registry

app = Typer()


rpath = None
if _path := getenv("NOO_REGISTRY_PATH"):
    rpath = Path(_path)

del _path

reg = Registry(rpath)

del rpath


@app.command("add")
def add(name: str, ref: str) -> None:
    reg.set_item(name, Path(ref))

    echo(f"Registered {name} as {ref}")


@app.command("remove")
def remove(name: str) -> None:
    try:
        reg.del_item(name)
    except KeyError:
        echo(f"No such key: {name}")
        return

    echo(f"Unregistered {name}")


@app.command("import")
def import_(file: str) -> None:
    data = loads(Path(file).read_text())

    for key, value in data.items():
        reg.set_item(key, str(Path(value).absolute()))

    echo(f"Imported {len(data)} items")


@app.command("export")
def export(file: str = "export.json") -> None:
    data = reg.read()

    Path(file).write_text(dumps(data, indent=2))

    echo(f"Exported {len(data)} items")



from __future__ import annotations

from re import compile

VAR = compile(r"\$\$(?P<namespace>[a-zA-Z_]+):(?P<name>[a-zA-Z_]+)")


def replace(text: str, src: str, dest: str, variables: dict[str, dict[str, str | int]]) -> str:
    resolved_dest = dest

    for ns, var in VAR.findall(dest):
        if ns not in ("noo", "var"):
            raise ValueError(f"Unknown namespace: {ns}")

        if var not in variables[ns]:
            raise ValueError(f"Unknown variable: {ns}:{var}")

        resolved_dest = resolved_dest.replace(f"$${ns}:{var}", str(variables[ns][var]))

    return text.replace(src, resolved_dest)


def format_vars(text: str, variables: dict[str, dict[str, str | int]]) -> str:
    for ns, var in VAR.findall(text):
        if ns not in ("noo", "var"):
            raise ValueError(f"Unknown namespace: {ns}")

        if var not in variables[ns]:
            raise ValueError(f"Unknown variable: {ns}:{var}")

        text = text.replace(f"$${ns}:{var}", str(variables[ns][var]))

    return text



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['noo',
 'noo.cli',
 'noo.cli.components',
 'noo.impl',
 'noo.impl.core',
 'noo.impl.models',
 'noo.impl.packager',
 'noo.impl.packager.runners',
 'noo.impl.resolvers',
 'noo.impl.utils']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'pydantic>=1.9.0,<2.0.0',
 'requests>=2.27.1,<3.0.0',
 'tomli>=1.2.3,<2.0.0',
 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['noo = noo:app']}

setup_kwargs = {
    'name': 'pynoo',
    'version': '1.4.0',
    'description': 'Easily create new projects.',
    'long_description': '# noo\n\nEasily create new projects.\n\n![Lint](https://github.com/py-noo/noo/actions/workflows/black.yml/badge.svg)\n\n## Installation\n\n```sh\npip install pynoo\n```\n\nor install from github\n\n```sh\npip install git+https://github.com/py-noo/noo\n```\n\n## Contributing\n\nSee [contributing](./.github/CONTRIBUTING.md).\n\n## Usage\n\n```sh\nnoo clone <name> <ref>\n```\n\n## Noofile Specification\n\n```yml\nname: str\nremote: str\nread: [Read]\nsteps: [Step]\n```\n\n| Field  | Type                | Description                            |\n|--------|---------------------|----------------------------------------|\n| name   | str                 | The name of the noofile definition     |\n| remote | str                 | The remote location of the template    |\n| read   | list[[Read](#read)] | The list of variables to read on setup |\n| steps  | list[[Step](#step)] | The list of steps to run               |\n\n### Read\n\n```yml\nname: str\nprompt: str\ndefault: ?str\n```\n\n| Field   | Type | Description                                     |\n|---------|------|-------------------------------------------------|\n| name    | str  | The name of the variable to read                |\n| prompt  | str  | The prompt to display when reading the variable |\n| default | ?str | An optional default value                       |\n\n### Step\n\nA step defines a single step in the process of setting up a project.\n\n```yml\nname: str\nactions: [Action]\nconditions: ?[Condition]\n```\n\n| Field      | Type                           | Description                                                 |\n|------------|--------------------------------|-------------------------------------------------------------|\n| name       | str                            | The name of the setup step                                  |\n| actions    | list[[Action](#action)]        | The list of actions in the step                             |\n| conditions | ?list[[Condition](#condition)] | An optional list of conditions required for the step to run |\n\n### Condition\n\nA condition is a function that must be true for a step to run.\n\n```yml\nop: eq | ne | gt | lt | ge | le\nvar: str\nvalue: str\n```\n\n| Field | Type                       | Description                            |\n|-------|----------------------------|----------------------------------------|\n| op    | Literal[[OpType](#optype)] | The operation to perform               |\n| var   | str                        | The variable to compare, i.e. noo:year |\n| value | str                        | The value to compare to                |\n\n### OpType\n\nList of operation types:\n\n| Name | Operation              |\n|------|------------------------|\n| eq   | Equals                 |\n| ne   | Not Equals             |\n| gt   | Greater Than           |\n| lt   | Less Than              |\n| ge   | Greater Than or Equals |\n| le   | Less Than or Equals    |\n\n### Action\n\nAn action defined a single action within a step. This is the base of all steps, for example replacing a string with a different given string.\n\n#### Replace action\n\nReplace actions are used to replace a specific string in a file. The `src` field specifies the string that should be replaced in the file, and the `dest` field specifices the string to replace it with. The `dest` field is formatted with defined variables.\n\nA list of files can be provided, and each file will have the same transform applied to them.\n\n```yml\n- action: replace\n  files: [str]\n  src: str\n  dest: str\n```\n\n#### Delete action\n\nDelete actions are used to delete files.\n\n```yml\n- action: delete\n  files: [str]\n```\n\n#### Create action\n\nCreate actions are used to create files. The `file` field specifies the file to be created, and the `content` field specifics the content to be placed into the file. The `content` field is formatted with defined variables.\n\n```yml\n- action: create\n  file: str\n  content: ?str\n```\n\n#### Rename action\n\nRename actions are used to rename files. The `file` field specifies the file to be renamed, and the `dest` field specifies the new name of the file. The `dest` field is formatted with defined variables.\n\n```yml\n- action: rename\n  file: str\n  dest: str\n```\n\n### Variables\n\nVariables are defined in the `read` section of the noofile. All variables set in the `read` section will be available in the `steps` section.\n\nVariables are used in the format `$${scope}:{name}`, for example `$$noo:year` or `$$var:author`. Variables with the `noo` scope are built into noo and will always be available. Variables with the `var` scope are defined in the `read` section.\n\nThe variables defined by noo are:\n\n- `noo:year` - The current year\n- `noo:month` - The current month\n- `noo:day` - The current day\n- `noo:hour` - The current hour\n- `noo:minute` - The current minute\n- `noo:second` - The current second\n- `noo:name` - The name of the project\n',
    'author': 'vcokltfre',
    'author_email': 'vcokltfre@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/py-noo',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from pathlib import Path
from typing import Type

from typer import echo
from yaml import safe_dump

from .runners import AutoPackagerRunner, JavaScriptRunner, PythonPoetryRunner

RUNNERS: list[Type[AutoPackagerRunner]] = [
    JavaScriptRunner,
    PythonPoetryRunner,
]


class Packager:
    def __init__(self, location: Path) -> None:
        runner = None

        for runner_type in RUNNERS:
            if runner_type.detect(location):
                runner = runner_type(location)
                echo(f"Using packager {runner.__class__.__name__}")
                break

        if runner is None:
            raise ValueError(f"No packager found for {location}")

        self.runner = runner

    def package(self, to: Path) -> None:
        spec = self.runner.package()

        to.write_text(safe_dump(spec.dict()))
